@@ -13,6 +13,7 @@ __all__ = ['array', 'amax','amin', 'max', 'min', 'maximum','minimum','nanmax','n
             'sqrt', 'square', 'absolute', 'abs', 'clip',
             'power',
             'add', 'divide', 'multiply', 'negative', 'subtract', 'reciprocal',
+            'nan_to_num',
             'dot', 'cumsum',
             'floor', 'ceil',
             'greater', 'greater_equal', 'less', 'less_equal', 'equal', 'not_equal',
@@ -223,7 +224,12 @@ class UnaryElemwise(ch.Ch):
         if wrt is self.x:
             result = self._d(self.x.r)
             return sp.diags([result.ravel()], [0]) if len(result)>1 else np.atleast_2d(result)
-    
+
+
+class nan_to_num(UnaryElemwise):
+    _r = lambda self, x : np.nan_to_num(x)
+    _d = lambda self, x : np.asarray(np.isfinite(x), np.float64)
+
 class reciprocal(UnaryElemwise):
     _r = np.reciprocal
     _d = lambda self, x : -np.reciprocal(np.square(x))
@@ -704,8 +710,8 @@ class dot(ch.Ch):
     
     def compute_d1(self):
         # To stay consistent with numpy, we must upgrade 1D arrays to 2D
-        ar = row(self.a.r) if len(self.a.r.shape)<2 else self.a.r
-        br = col(self.b.r) if len(self.b.r.shape)<2 else self.b.r
+        ar = row(self.a.r) if len(self.a.r.shape)<2 else self.a.r.reshape((-1, self.a.r.shape[-1]))
+        br = col(self.b.r) if len(self.b.r.shape)<2 else self.b.r.reshape((self.b.r.shape[0], -1))
 
         if ar.ndim <= 2:
             return sp.kron(sp.eye(ar.shape[0], ar.shape[0]),br.T)
@@ -715,8 +721,8 @@ class dot(ch.Ch):
     def compute_d2(self):
         
         # To stay consistent with numpy, we must upgrade 1D arrays to 2D
-        ar = row(self.a.r) if len(self.a.r.shape)<2 else self.a.r
-        br = col(self.b.r) if len(self.b.r.shape)<2 else self.b.r
+        ar = row(self.a.r) if len(self.a.r.shape)<2 else self.a.r.reshape((-1, self.a.r.shape[-1]))
+        br = col(self.b.r) if len(self.b.r.shape)<2 else self.b.r.reshape((self.b.r.shape[0], -1))
 
         if br.ndim <= 1:
             return self.ar
